@@ -1,10 +1,9 @@
 """This defines a basic wsgi application that provides RESTful routing."""
 
 from werkzeug.exceptions import HTTPException
-from werkzeug.exceptions import NotFound
 from werkzeug.exceptions import MethodNotAllowed
 from werkzeug.exceptions import InternalServerError
-from werkzeug.wrappers import Request
+from werkzeug.wrappers import Request, BaseRequest, BaseResponse
 
 
 class RestApplication(object):
@@ -43,46 +42,61 @@ class RestApplication(object):
         urls = self.url_map.bind_to_environ(request.environ)
 
         # Attempt to process pre-request hooks.
+        print "PROCESSING REQUEST HOOKS"
         try:
 
-            self.process_request_hooks(request)
+            request = self.process_request_hooks(request)
 
         except HTTPException as e:
 
+            print "REQUEST HOOK HTTP EXCEPTION"
+
             response = e
 
-            self.process_response_hooks(request, response)
+            response = self.process_response_hooks(request, response)
 
             return response
 
         except Exception as e:
 
+            print "REQUEST HOOK PY EXCEPTION"
+
             response = InternalServerError(description=unicode(e))
 
+            response = self.process_response_hooks(request, response)
+
+            return response
+
         # The endpoint in this case is an object constructor which is why the
         # variable is captialzied.
         # This call automatically raises a `NotFound` exception if there is no
         # match found.
+        print "MATCHING URLS"
         try:
 
             Endpoint, kwargs = urls.match()
 
         except HTTPException as e:
 
+            print "URL HTTP EXCEPTION"
+
             response = e
 
-            self.process_response_hooks(request, response)
+            print "PROCESSING RESPONSE HOOKS"
+            response = self.process_response_hooks(request, response)
 
             return response
 
         handler_endpoint = Endpoint()
 
+        print "FINDING METHOD"
         try:
 
             handler_method = getattr(handler_endpoint, verb)
 
         except AttributeError:
 
+            print "METHOD NOT FOUND"
             # If method not found then generate a list of allowed methods.
             # The list of allowed methods is required to satisfy the HTTP
             # standard for Method Now Allowed. Currently only matches GET,
@@ -97,23 +111,30 @@ class RestApplication(object):
                                                            ]
                                                        ])
 
-            self.process_response_hooks(request, response)
+            print "PROCESSING POST HOOKS"
+            response = self.process_response_hooks(request, response)
 
             return response
 
+        print "EXECUTING METHOD"
         try:
 
             response = handler_method(request, **kwargs)
 
         except HTTPException as e:
 
+            print "EXECUTION WAS HTTP EXCEPTION"
+
             response = e
 
         except Exception as e:
 
+            print "EXECUTION WAS PY EXCEPTION"
+
             response = InternalServerError(description=unicode(e))
 
-        self.process_response_hooks(request, response)
+        print "PROCESSING RESPONSE HOOKS"
+        response = self.process_response_hooks(request, response)
 
         return response
 
@@ -121,10 +142,22 @@ class RestApplication(object):
 
         for hook in self.pre_hooks:
 
-            hook(request)
+            result = hook(request)
+
+            if isinstance(result, BaseRequest):
+
+                request = result
+
+        return request
 
     def process_response_hooks(self, request, response):
 
         for hook in self.post_hooks:
 
-            hook(request, response)
+            result = hook(request, response)
+
+            if isinstance(result, BaseResponse):
+
+                response = result
+
+        return response
